/**\
|**|	App.c
\**/

/**\
|**|	compiler directives
\**/

#define SystemSevenOrLater		1

/**\
|**|	Includes
\**/

#include <Devices.h>
#include <Fonts.h>
#include <CodeFragments.h>
#include <Dialogs.h>
#include <DiskInit.h>
#include <LowMem.h>
#include <fp.h>
#include <PLStringFuncs.h>
#include <Sound.h>
#include <Traps.h>
#include <Threads.h>

#include <stdio.h>
#include <string.h>

#include "HappyTrails.h"

/**\
|**|	typedefs, structs, enums, etc.
\**/

enum
{
	mAppleMenu = 128,
		iAboutBox = 1,
	mFileMenu = 129,
		iQuit = 1,
	mTasksMenu = 130,
	mWeightMenu = 131
};

/**\
|**|	private (local) function prototypes
\**/

static OSErr InitMac(void);
static void Command(long ms);
static void DoEvent(const EventRecord *pEventPtr);
static void DoNullEvent(const EventRecord *pEventPtr);
static void DoMouseEvent(const EventRecord *pEventPtr);
static void DoKeyEvent(char key,SInt16 modifiers);
static void DoUpdateEvent(WindowPtr updateWindowP);
static void DoActivateEvent(WindowPtr updateWindowP);
static void DoDiskEvent(long message);
static void DoOSEvent(long message);
static void doContentClick(WindowPtr pWindowPtr,const EventRecord *pEventPtr);

static Boolean SetUpMenuBar(void);
static void AdjustMenuItems(void);

/**\
|**|	globals & constants
\**/

WindowPtr	gWindowPtr = nil;
Rect		gWindowRect;
Boolean		gInBackGround = false;
Rect 		gMenuRect = {0,0,0,0};

static Boolean gQuitFlag = false;

/**\
|**|	main
\**/

void main(void)
{
	if (!InitMac() && SetUpMenuBar() && !HT_Init(9))
	{
		do
		{
			EventRecord event;
			InitCursor();
			WaitNextEvent(everyEvent,&event,-1,nil);
			DoEvent(&event);
		}
		while (!gQuitFlag);

		HT_Term();
	}
}

/**\
|**|	Initialize toolboxes
\**/

static OSErr InitMac(void)
{
	OSErr		error;
	SysEnvRec	theWorld;
		
	//
	//	Test the computer to be sure we can do color.  
	//	If not we would crash, which would be bad.  
	//	If we cant run, just beep and exit.
	//

	error = SysEnvirons(1, &theWorld);
	if (theWorld.hasColorQD == false)
	{
		SysBeep(50);
		ExitToShell();					// If no color QD, we must leave.
	}
	
	MaxApplZone();
	InitGraf(&(qd.thePort));
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);

	InitContextualMenus();

	//
	//	Make a new window for drawing in, and it must be a color window.  
	//	The window is full screen size, made smaller to make it more visible.
	//

	gWindowRect = qd.screenBits.bounds;
//	InsetRect(&gWindowRect, 20, 20);
	gWindowRect.top += LMGetMBarHeight() + 1;

	gWindowPtr = NewCWindow(nil, &gWindowRect, "\pHappy Trails!", true, documentProc, 
						(WindowPtr) -1, false, 0);

	OffsetRect(&gWindowRect, -gWindowRect.left, -gWindowRect.top);

	SetPort(gWindowPtr);					// set window to current graf port
	TextSize(kTextSize);					// smaller font for drawing.

	return noErr;
}

/**\
|**|	Menu hooks
\**/

static pascal short MyMBarHook(Rect *menuRect)
{
//	RgnHandle tempRgn = NewRgn();

	gMenuRect = *menuRect;

//	RectRgn(tempRgn,menuRect);
//	CalcVisBehind(LMGetWindowList(),tempRgn);
//	InvertRgn(tempRgn);

	//YieldToAnyThread();
//	DisposeRgn(tempRgn);
	return 0;
}

static pascal void MyMenuHook(void)
{	
//	RgnHandle tempRgn = NewRgn();
//	RectRgn(tempRgn,&gMenuRect);
//	CalcVisBehind(LMGetWindowList(),tempRgn);
//	InvertRgn(tempRgn);

	//YieldToAnyThread();
//	DisposeRgn(tempRgn);
}

/**\
|**|	Setup menu bar
\**/

static Boolean SetUpMenuBar(void)
{
	Handle mBar = GetNewMBar(128);	// handle to menu bar resource

	if (!ResError() && mBar)
	{
		SetMenuBar(mBar);
		AppendResMenu(GetMenuHandle(mAppleMenu),'DRVR');
		DrawMenuBar();
		ReleaseResource(mBar);
		AdjustMenuItems();

		LMSetMBarHook(NewMBarHookProc(MyMBarHook));
		LMSetMenuHook(NewMenuHookProc(MyMenuHook));

		return true;
	}
	return false;
}

/**\
|**|	Adjust menu items
\**/

static void AdjustMenuItems(void)
{
	MenuHandle tMenuHdl = GetMenuHandle(mTasksMenu);
	if (tMenuHdl)
	{
		UInt16 index,count = CountMItems(tMenuHdl);
		for (index = 1;index <= count;index++)
			CheckItem(tMenuHdl, index, index == gNumTasks);
	}
}

/**\
|**|	Do menu command
\**/

static void Command(long ms)
{
	short	menuID = ms >> 16,
			menuItem = ms & 0xFFFF;

	switch (menuID)
	{
		case	mAppleMenu:
			switch (menuItem)
			{
				case iAboutBox:		// Bring up alert for About.
					SysBeep(15);
					break;
				default:			// All non-About items in this menu are DAs.
					{
						Str255	daName;
						GetMenuItemText(GetMenuHandle(menuID),menuItem,daName);
						OpenDeskAcc(daName);
					}
					break;
			}
			break;
		case	mFileMenu:
			switch (menuItem)
			{
				case iQuit:
					gQuitFlag = true;
					break;
				default:
					SysBeep(15);
					break;
			}
			break;
		case	mTasksMenu:
			HT_Term();
			HT_Init(menuItem);
			break;
		case	mWeightMenu:		// Weight Menu
			break;
	}
}

/**\
|**|	Do event
\**/

static void DoEvent(const EventRecord *pEventPtr)
{
	switch (pEventPtr->what)
	{
		case nullEvent:
			DoNullEvent(pEventPtr);
			break;
		case mouseDown:
			DoMouseEvent(pEventPtr);
			break;
		case keyDown:
		case autoKey:
dokey:
			DoKeyEvent((char)(pEventPtr->message & charCodeMask),pEventPtr->modifiers);
			break;
		case updateEvt:
			DoUpdateEvent((WindowPtr)pEventPtr->message);
			break;
		case diskEvt:
			DoDiskEvent(pEventPtr->message);
			break;
		case osEvt:
			DoOSEvent(pEventPtr->message);
			break;
		case kHighLevelEvent:
			AEProcessAppleEvent(pEventPtr);
			break;
		default:
			break;
	}
}

/**\
|**|	Do null event
\**/

static void DoNullEvent(const EventRecord *pEventPtr)
{
	(pEventPtr);	// #pragma unused (pEventPtr)
	if (gWindowPtr)
	{
		Rect	tRect = {40,20,52,128};
		const RGBColor blackRGBColor = {0,0,0}, yellowRGBColor = {255,255,0};
		GrafPtr savePort;

		GetPort(&savePort);
		SetPort(gWindowPtr);

		HT_DoNull();

		SetPort(savePort);
	}
}

/**\
|**|	Do mousedown event
\**/

static void DoMouseEvent(const EventRecord *pEventPtr)
{
	Boolean tInBackGround = gInBackGround;
	WindowPtr			window;
	short				part;

	gInBackGround = true;	// suspend drawing while menus are displayed

	part = FindWindow(pEventPtr->where,&window);

	if (part != inContent)
		SetCursor(&qd.arrow);

	switch(part)
	{
		case inContent:
			{
				if (window != FrontWindow())
					SelectWindow(window);
				else
					HT_DoClick(pEventPtr);
			}
			break;

		case inMenuBar:		// Process mouse menu command (if any).
			{
				long ms;

				AdjustMenuItems();
				ms = MenuSelect(pEventPtr->where);
				if (ms)
					Command(ms);
				HiliteMenu(0);		// Unhighlight what MenuSelect hilited.
			}
			break;

		case inSysWindow:	// Let the system handle the mouseDown.
			SystemClick(pEventPtr,window);
			break;

		default:
			break;
	}
	gInBackGround = tInBackGround;

	SetRect(&gMenuRect, 0, 0, 0, 0);
}

/**\
|**|	Do key events
\**/

static void DoKeyEvent(char key,SInt16 modifiers)
{
	if ((modifiers & cmdKey) != 0)
	{
		AdjustMenuItems();
		Command(MenuKey(key));
		HiliteMenu(0);		// Unhighlight what MenuSelect hilited.
	}
}

/**\
|**|	Do update events
\**/

static void DoUpdateEvent(WindowPtr updateWindowP)
{
	GrafPtr savePort;

	GetPort(&savePort);
	SetPort(updateWindowP);
	BeginUpdate(updateWindowP);

	UpdateControls(updateWindowP,updateWindowP->visRgn);

	DrawGrowIcon(updateWindowP);
	EndUpdate(updateWindowP);

	SetPort(savePort);
}

/**\
|**|	Do disk events
\**/

static void DoDiskEvent(long message)
{
	Point dialogLocation = {100,100};

	if ((message & 0xFFFF0000) != noErr)
	{
		DIBadMount(dialogLocation,message);
	}
}

/**\
|**|	Do OS events
\**/

static void DoOSEvent(long message)
{
	if ((message >> 24) == suspendResumeMessage)
	{
		if ((message & resumeFlag) != 0)
			gInBackGround = false;
		else
			gInBackGround = true;
	}
}

